/****************************************************************************
**
** Copyright (C) 2010 Nokia Corporation and/or its subsidiary(-ies).
** All rights reserved.
** Contact: Nokia Corporation (qt-info@nokia.com)
**
** This file is part of the examples of the Qt Toolkit.
**
** $QT_BEGIN_LICENSE:LGPL$
** Commercial Usage
** Licensees holding valid Qt Commercial licenses may use this file in
** accordance with the Qt Commercial License Agreement provided with the
** Software or, alternatively, in accordance with the terms contained in
** a written agreement between you and Nokia.
**
** GNU Lesser General Public License Usage
** Alternatively, this file may be used under the terms of the GNU Lesser
** General Public License version 2.1 as published by the Free Software
** Foundation and appearing in the file LICENSE.LGPL included in the
** packaging of this file.  Please review the following information to
** ensure the GNU Lesser General Public License version 2.1 requirements
** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
**
** In addition, as a special exception, Nokia gives you certain additional
** rights.  These rights are described in the Nokia Qt LGPL Exception
** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
**
** GNU General Public License Usage
** Alternatively, this file may be used under the terms of the GNU
** General Public License version 3.0 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.  Please review the following information to
** ensure the GNU General Public License version 3.0 requirements will be
** met: http://www.gnu.org/copyleft/gpl.html.
**
** If you have questions regarding the use of this file, please contact
** Nokia at qt-info@nokia.com.
** $QT_END_LICENSE$
**
****************************************************************************/

#ifndef CODEEDITOR_H
#define CODEEDITOR_H

#include <QPlainTextEdit>
#include <QObject>
#include "highlighter.h"

QT_BEGIN_NAMESPACE
class QPaintEvent;
class QResizeEvent;
class QSize;
class QCompleter;
class QWidget;
class chGL;
QT_END_NAMESPACE

class LineNumberArea;
class AfixHighlightArea;//ein versuch

//![codeeditordefinition]

class CodeEditor : public QPlainTextEdit {
    Q_OBJECT

public:
    CodeEditor(QWidget *parent = 0);
    ~CodeEditor();
    Highlighter *highlighter;
    void setCompleter(QCompleter *c);
    QCompleter *completer() const;
    QStringList blocks;
    QList<int> errorInLine;
 
    void lineNumberAreaPaintEvent(QPaintEvent *event);
    void afixHighlightAreaPaintEvent(QPaintEvent *event);
    int lineNumberAreaWidth();
    void diffTo(QString org);
    QList<int> parenthesis;
    QList<int> parenthesis2;
    QList<bool> parenthesis3;
    QList<bool> comment;
    QList<double> fv;
    int fvMinLine;
signals:
    void findInStructure(const QString &);
    void electInStructure(const QString &);
    void search(const QString&);
    void deleteSelected();

protected:
    void resizeEvent(QResizeEvent *event);
    bool viewportEvent(QEvent *event);
    void keyPressEvent(QKeyEvent *e);
    void focusInEvent(QFocusEvent *e);
    void contextMenuEvent(QContextMenuEvent *event);
public slots:
    void updateLineNumberAreaWidth(int newBlockCount);
//    void highlightCurrentLine();
//    void highlightSelection(const QString &find);
    void updateLineNumberArea(const QRect &, int);
    void updateAfix();
    void insertCompletion(const QString &completion);
    void searchInStructure();
    void selectInStructure();
    void lineNumberToggled(QMouseEvent *event);
    void remark(int line);
    void toggleRemaks();
    void setgl(chGL *g){
      chgl=g;
    }

private:
    QString textUnderCursor() const;
    QWidget *lineNumberArea;
    QCompleter *c;
    QWidget *afixHighlightArea;
    chGL *chgl;
};

/////////////////////////////////////////////////
class AfixHighlightArea : public QWidget{
public:
  AfixHighlightArea(CodeEditor *editor) : QWidget(editor){
    codeEditor = editor;
  }
  QSize sizeHint() const {
    return QSize(5,0);
  }
protected:
  void paintEvent(QPaintEvent *event){
    codeEditor->afixHighlightAreaPaintEvent(event);
  }

private:
  CodeEditor *codeEditor;
};
/////////////////////////////////////////////////

class LineNumberArea : public QWidget{
public:
  LineNumberArea(CodeEditor *editor) : QWidget(editor) {
    codeEditor = editor;
  }

  QSize sizeHint() const {
    return QSize(codeEditor->lineNumberAreaWidth(), 0);
  }

protected:
  void paintEvent(QPaintEvent *event) {
    codeEditor->lineNumberAreaPaintEvent(event);
  }
  void mousePressEvent ( QMouseEvent * event ){
    codeEditor->lineNumberToggled(event);
  }

private:
  CodeEditor *codeEditor;
};

#endif
